class TermColor:
    ERROR_HIGHLIGHT = "\u001b[41m"  # red background
    OK_HIGHLIGHT = "\u001b[42m"  # green background
    WARNING_HIGHLIGHT = "\u001b[43m"  # yellow background
    ERROR = "\033[38;5;160m"  # red
    OK = "\033[38;5;46m"  # green
    WARNING = "\033[38;5;220m"
    ENDC = "\033[0m"
